------------------------------------------------------
-- EnchantSeller.lua
-- Version 1500.1
------------------------------------------------------

FES_Config = { };
FES_Config.Enabled = true;

FES_EnchantHistory = { };
-- Has the following internal structure:
--		REALM = {
--			PLAYER = { -- 
--				ENCHANT_NAME = 
--					PRICE, -- price can be number (copper) or list of item names (starting with number if money is also traded)
--			}
--		}

-- Variables
FES_Trade = { };
FES_Realm = nil;
FES_Player = nil;

function FES_CraftFrame_SetSelection(id)
	FES_Orig_CraftFrame_SetSelection(id);
	
	if ( not id ) then
		return;
	end
	local name, rank, maxRank = GetCraftDisplaySkillLine();
	if not (name) then
		return;
	end
	local craftName, craftSubSpellName, craftType, numAvailable, isExpanded, trainingPointCost, requiredLevel = GetCraftInfo(id);
	if ( trainingPointCost and trainingPointCost > 0 ) then
		return;
	end
	if ( craftType == "header" ) then
		return;
	end
	
	local allCount, myCount, allMeanPrice, allMedianPrice = FES_EnchantPriceInfo(craftName);
	local costText;
	if (allCount > 0) then
		costText = "Median price: " .. FES_GetTextGSC(allMedianPrice);
		costText = costText .. " Average: " .. FES_GetTextGSC(allMeanPrice);
		costText = costText .. "\n"..GRAY_FONT_COLOR_CODE.."(Sold for cash "..allCount.." times, "..myCount.." by me)"..FONT_COLOR_CODE_CLOSE;
	else
		costText = GRAY_FONT_COLOR_CODE.."No trade data"..FONT_COLOR_CODE_CLOSE;
	end
	CraftCost:SetTextColor(1.0, 1.0, 0.6);
	CraftCost:SetJustifyH("LEFT");
	CraftCost:SetText(costText);
	CraftCost:Show();
end

function FES_OnLoad()

	-- Register for Events
	this:RegisterEvent("TRADE_ACCEPT_UPDATE");
	this:RegisterEvent("TRADE_CLOSED");
	this:RegisterEvent("TRADE_TARGET_ITEM_CHANGED");
	this:RegisterEvent("TRADE_PLAYER_ITEM_CHANGED");
	
	-- Register Slash Commands
	SLASH_FES1 = "/enchantseller";
	SLASH_FES2 = "/es";
	SlashCmdList["FES"] = function(msg)
		FES_ChatCommandHandler(msg);
	end
	
	-- Override for displaying info in CraftFrame
	FES_Orig_CraftFrame_SetSelection = CraftFrame_SetSelection;
	CraftFrame_SetSelection = FES_CraftFrame_SetSelection;
	
	GFWUtils.Print(GFW_COLOR_START.."Fizzwidget Enchant Seller initialized!"..FONT_COLOR_CODE_CLOSE);

end

function FES_CheckSetup()

	if (FES_Realm == nil) then
		FES_Realm = GetCVar("realmName");
	end
	if (FES_Player == nil) then
		FES_Player = UnitName("player");
	end
	if (FES_EnchantHistory == nil) then
		FES_EnchantHistory = { };
	end

end

function FES_OnEvent(event, arg1)

	if ( event == "TRADE_ACCEPT_UPDATE") then
	
		local name, texture, numItems, isUsable, _, enchantment = GetTradePlayerItemInfo(7);
		FES_Trade.PlayerEnchant = enchantment;		
		local name, texture, numItems, isUsable, _, enchantment = GetTradeTargetItemInfo(7);
		FES_Trade.TargetEnchant = enchantment;

		FES_Trade.PlayerMoney = GetPlayerTradeMoney();
		FES_Trade.TargetMoney = GetTargetTradeMoney();
		FES_Trade.PlayerItems= { };
		FES_Trade.TargetItems= { };
		
		for i=1, 6 do
			local name, texture, numItems, isUsable, enchantment = GetTradePlayerItemInfo(i);
			if (name) then
				if (numItems > 1) then
					table.insert(FES_Trade.PlayerItems, numItems.."x["..name.."]");
				else
					table.insert(FES_Trade.PlayerItems, "["..name.."]");
				end
			end
			local name, texture, numItems, isUsable, enchantment = GetTradeTargetItemInfo(i);
			if (name) then
				if (numItems > 1) then
					table.insert(FES_Trade.TargetItems, numItems.."x["..name.."]");
				else
					table.insert(FES_Trade.TargetItems, "["..name.."]");
				end
			end
		end
		return;
		
	elseif ( event == "TRADE_CLOSED") then
		
		if (FES_Trade.PlayerEnchant == nil and FES_Trade.TargetEnchant == nil) then 
			-- nothing is being enchanted, so we don't care about saving info; clear state until another trade happens
			FES_Trade = { };
			return; 
		end
		
		FES_CheckSetup();
		if (FES_EnchantHistory[FES_Realm] == nil) then
			FES_EnchantHistory[FES_Realm] = { };
		end
		
		if (FES_Trade.TargetEnchant) then
		
			if (FES_EnchantHistory[FES_Realm][FES_Player] == nil) then
				FES_EnchantHistory[FES_Realm][FES_Player] = { };
			end
			if not (FES_EnchantHistory[FES_Realm][FES_Player][FES_Trade.TargetEnchant]) then
				FES_EnchantHistory[FES_Realm][FES_Player][FES_Trade.TargetEnchant] = { };
			end
			
			if (table.getn(FES_Trade.TargetItems) > 0 and FES_Trade.TargetMoney > 0) then
				table.insert(FES_Trade.TargetItems, 1, FES_Trade.TargetMoney);
				table.insert(FES_EnchantHistory[FES_Realm][FES_Player][FES_Trade.TargetEnchant], FES_Trade.TargetItems);
			elseif(table.getn(FES_Trade.TargetItems) > 0) then
				table.insert(FES_EnchantHistory[FES_Realm][FES_Player][FES_Trade.TargetEnchant], FES_Trade.TargetItems);
			elseif(FES_Trade.TargetMoney > 0) then
				table.insert(FES_EnchantHistory[FES_Realm][FES_Player][FES_Trade.TargetEnchant], FES_Trade.TargetMoney);
			else
				-- enchanting for free!
				table.insert(FES_EnchantHistory[FES_Realm][FES_Player][FES_Trade.TargetEnchant], 0);
			end
			
			FES_Trade = { };
			return;
		end
		if (FES_Trade.PlayerEnchant) then
		
			if (FES_EnchantHistory[FES_Realm]["_RECEIVED"] == nil) then
				FES_EnchantHistory[FES_Realm]["_RECEIVED"] = { };
			end
			if not (FES_EnchantHistory[FES_Realm]["_RECEIVED"][FES_Trade.PlayerEnchant]) then
				FES_EnchantHistory[FES_Realm]["_RECEIVED"][FES_Trade.PlayerEnchant] = { };
			end
			if (table.getn(FES_Trade.PlayerItems > 0) and FES_Trade.PlayerMoney > 0) then
				table.insert(FES_Trade.PlayerItems, 1, FES_Trade.PlayerMoney);
				table.insert(FES_EnchantHistory[FES_Realm]["_RECEIVED"][FES_Trade.PlayerEnchant], FES_Trade.PlayerItems);
			elseif(table.getn(FES_Trade.PlayerItems) > 0) then
				table.insert(FES_EnchantHistory[FES_Realm]["_RECEIVED"][FES_Trade.PlayerEnchant], FES_Trade.PlayerItems);
			elseif(FES_Trade.PlayerMoney > 0) then
				table.insert(FES_EnchantHistory[FES_Realm]["_RECEIVED"][FES_Trade.PlayerEnchant], FES_Trade.PlayerMoney);
			else
				-- enchanting for free!
				table.insert(FES_EnchantHistory[FES_Realm]["_RECEIVED"][FES_Trade.PlayerEnchant], 0);
			end
			
			FES_Trade = { };
			return;
		end
		
		return;
			
	end
	
end

function FES_ChatCommandHandler(msg)

	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		GFWUtils.Print(GFW_COLOR_START.."Fizzwidget Enchant Seller:"..FONT_COLOR_CODE_CLOSE);
		GFWUtils.Print(GFW_COLOR_START.."/enchantseller /es <command>"..FONT_COLOR_CODE_CLOSE);
		GFWUtils.Print("- "..GFW_COLOR_START.."help"..FONT_COLOR_CODE_CLOSE.." - Print this helplist.");
		GFWUtils.Print("- "..GFW_COLOR_START.."on|off"..FONT_COLOR_CODE_CLOSE.." - Toggle storing info when you trade enchants.");
		GFWUtils.Print("- "..GFW_COLOR_START.."status"..FONT_COLOR_CODE_CLOSE.." - Check current settings.");
		GFWUtils.Print("- "..GFW_COLOR_START.."reset"..FONT_COLOR_CODE_CLOSE.." - Reset to default settings.");
		GFWUtils.Print("- "..GFW_COLOR_START.."price <enchant name>"..FONT_COLOR_CODE_CLOSE.." - Show price summary for an enchantment.");
		return;
	end
	
	-- Check Status
	if ( msg == "status" ) then
		if (FES_Config.Enabled) then
			GFWUtils.Print(GFW_COLOR_START.."Enchant Seller is gathering info when you trade."..FONT_COLOR_CODE_CLOSE);
		else
			GFWUtils.Print(GFW_COLOR_START.."Enchant Seller is NOT gathering info when you trade."..FONT_COLOR_CODE_CLOSE);
		end
		return;
	end

	-- Turn trade info gathering on
	if ( msg == "gatherinfo on" ) then
		FES_Config.Enabled = true;
		GFWUtils.Print(GFW_COLOR_START.."Enchant Seller is gathering info when you trade."..FONT_COLOR_CODE_CLOSE);
		return;
	end
	
	-- Turn trade info gathering Off
	if ( msg == "gatherinfo off" ) then
		FES_Config.Enabled = false;
		GFWUtils.Print(GFW_COLOR_START.."Enchant Seller is NOT gathering info when you trade."..FONT_COLOR_CODE_CLOSE);
		return;
	end
	
	-- Reset Variables
	if ( msg == "reset" ) then
		FES_Config.Enabled = true;
		GFWUtils.Print(GFW_COLOR_START.."Enchant Seller configuration reset."..FONT_COLOR_CODE_CLOSE);
		FES_ChatCommandHandler("status");
		return;
	end
	
	if ( string.sub(msg, 1, 5) == "price" ) then
		local enchantName = string.sub(msg, 7);
		local allCount, myCount, allMeanPrice, allMedianPrice = FES_EnchantPriceInfo(enchantName);
		if (allCount > 0) then
			if (myCount > 0) then
				GFWUtils.Print(GFW_COLOR_START..enchantName..FONT_COLOR_CODE_CLOSE..": "..allCount.." sales (".. myCount.." by "..FES_Player..").");
			else
				GFWUtils.Print(GFW_COLOR_START..enchantName..FONT_COLOR_CODE_CLOSE..": "..allCount.." sales.");
			end
			GFWUtils.Print(GFW_COLOR_START.."Median price: "..FONT_COLOR_CODE_CLOSE..FES_GetGSC(allMedianPrice));
			GFWUtils.Print(GFW_COLOR_START.."Average price: "..FONT_COLOR_CODE_CLOSE..FES_GetGSC(allMeanPrice));
		else
			GFWUtils.Print("No sales data for "..GFW_COLOR_START..enchantName..FONT_COLOR_CODE_CLOSE..".");
		end
		return;
	end
		
end

function FES_EnchantPriceInfo(enchantName)
	
	FES_CheckSetup();
	
	local myPrices = FES_MyPricesForEnchant(enchantName);
	local myCount = table.getn(myPrices);
	local allPrices = FES_AllRealmPricesForEnchant(enchantName);
	local allCount = table.getn(allPrices);
	
	local allMeanPrice = GFWTable.Mean(allPrices);
	local allMedianPrice = GFWTable.Median(allPrices);
	
	return allCount, myCount, allMeanPrice, allMedianPrice;
end

function FES_MyPricesForEnchant(enchantName)
	local myCashEnchants = { };
	if (FES_EnchantHistory[FES_Realm] and FES_EnchantHistory[FES_Realm][FES_Player]) then
		if (FES_EnchantHistory[FES_Realm][FES_Player][enchantName]) then
			for anIndex, aTrade in FES_EnchantHistory[FES_Realm][FES_Player][enchantName] do
				if (tonumber(aTrade) and aTrade > 0) then
					table.insert(myCashEnchants, aTrade);
				end
			end
		end
	end
	return myCashEnchants;
end

function FES_AllRealmPricesForEnchant(enchantName)
	
	local allCashEnchants = { };
	if (FES_EnchantHistory[FES_Realm]) then
		for aPlayer, playerEnchants in FES_EnchantHistory[FES_Realm] do
			if (playerEnchants[enchantName]) then
				for anIndex, aTrade in playerEnchants[enchantName] do
					if (tonumber(aTrade) and aTrade > 0) then
						table.insert(allCashEnchants, aTrade);
					end
				end
			end
		end
	end
	return allCashEnchants;
end

-- Thanks to Auctioneer for cash-formatting functions!

-- calculate the gold, silver, and copper values based the ammount of copper
function FES_GetGSC(money)
	if (money == nil) then money = 0; end
	local g = math.floor(money / 10000);
	local s = math.floor((money - (g*10000)) / 100);
	local c = math.floor(money - (g*10000) - (s*100));
	return g,s,c;
end

-- formats money text by color for gold, silver, copper
function FES_GetTextGSC(money)
    local GSC_GOLD="ffd100";
    local GSC_SILVER="e6e6e6";
    local GSC_COPPER="c8602c";
    local GSC_START="|cff%s%d|r";
    local GSC_PART=".|cff%s%02d|r";
    local GSC_NONE="|cffa0a0a0none|r";

	local g, s, c = FES_GetGSC(money);

	local gsc = "";
	if (g > 0) then
		gsc = format(GSC_START, GSC_GOLD, g);     
		if (s > 0) then
			gsc = gsc..format(GSC_PART, GSC_SILVER, s);
		end
	elseif (s > 0) then
		gsc = format(GSC_START, GSC_SILVER, s).."s";
		if (c > 0) then
			gsc = gsc..format(GSC_PART, GSC_COPPER, c);
		end
	elseif (c > 0) then
		gsc = gsc..format(GSC_START, GSC_COPPER, c);
	else
		gsc = GSC_NONE;
	end

	return gsc;
end

