------------------------------------------------------
-- GFWTable.lua
-- Version 1500.1
-- Utilities for manipulating tables 
------------------------------------------------------

GFWTable = {

-- Mean: returns the mean value of a table of numbers
Mean = function(aTable)
	return GFWTable.Sum(aTable) / table.getn(aTable);
end;

-- Sum: returns the sum of a table of numbers
Sum = function(aTable)
	local sum = 0;
	for i=1, table.getn(aTable) do
		if (tonumber(aTable[i])) then
			sum = sum + aTable[i];
		end
	end
	return sum;
end;

-- Median: returns the median value (most useful in a table of numbers, but usable in any sorted table)
Median = function(aTable)
	if (aTable == nil or table.getn(aTable) == 0) then 
		return nil; 
	end
	if (table.getn(aTable) == 1) then
		return aTable[1];
	end
	local sortedTable = GFWTable.Copy(aTable); -- leave the original table in whatever order it's in
	table.sort(sortedTable);
	local count = table.getn(sortedTable);
	local median;
	if (math.mod(count, 2) == 0) then -- even table size
		local middleIndex1 = count / 2;
		local middleIndex2 = middleIndex1 + 1;
		median = (sortedTable[middleIndex1] + sortedTable[middleIndex2]) / 2; --average the two middle values
	else -- the table size is odd
		local trueMiddleIndex = (tableSize + 1) / 2; -- calculate the middle index
		median = sortedTable[trueMiddleIndex];
	end
	return median;
end;

-- Merge: returns the union of two tables (without repeated elements)
Merge = function(table1, table2)
	local newTable = { };
	if (table1) then
		for i=1, table.getn(table1) do
			table.insert(newTable, table1[i]);
		end
	end
	if (table2) then
		for i=1, table.getn(table2) do
			if (GFWTable.IndexOf(newTable, table2[i]) == 0) then
				table.insert(newTable, table2[i]);
			end
		end
	end
	return newTable;
end;

-- Diff: returns the difference of two tables (those elements which occur in either but not both)
Diff = function(table1, table2)
	local newTable = { };
	if (table1 == nil) then
		table1 = {};
	end
	if (table2 == nil) then
		table2 = {};
	end
	for i=1, table.getn(table1) do
		if (GFWTable.IndexOf(table2, table1[i]) == 0) then
			table.insert(newTable, table1[i]);
		end
	end
	for i=1, table.getn(table2) do
		if (GFWTable.IndexOf(table1, table2[i]) == 0) then
			table.insert(newTable, table2[i]);
		end
	end
	return newTable;
end;

-- Subtract: returns a table containing those items in table1 not present in table2
Subtract = function(table1, table2)
	local newTable = { };
	if (table1 == nil or table.getn(table1) == 0) then return newTable; end
	if (table2 == nil or table.getn(table2) == 0) then return table1; end
	for i=1, table.getn(table1) do
		if (GFWTable.IndexOf(table2, table1[i]) == 0) then
			table.insert(newTable, table1[i]);
		end
	end
	return newTable;
end;

-- IndexOf: returns the index (1-based) of an item in a table
IndexOf = function(aTable, item)
	for i=1, table.getn(aTable) do
		if (item == aTable[i]) then
			return i;
		end
	end
	return 0;
end;

-- Copy: copies one table's elements into a new table (useful if you want to change them while preserving the first table). 
--       Not a deep copy.
Copy = function(aTable)
	local newTable = { };
	for i=1, table.getn(aTable) do
		newTable[i] = aTable[i];
	end
	return newTable;
end;

};
